#ifndef __ATUTILS__
#define __ATUTILS__

#ifndef __TYPES__
#include <Types.h>
#endif

#ifndef __APPLETALK__
#include <AppleTalk.h>
#endif

#ifndef __LISTS__
#include <Lists.h>
#endif

OSErr			ATInit(void);
	/*
	**	 Initialize ATUtils.
	**
	**	Currently does nothing.  Calling it allows the AppleTalk.c.o file to be
	**	needed by the link. */

OSErr			DoBuildZoneList(ListHandle listHndl);
	/*
	**	 Build a zone list into a list handle.
	**
	**	OUTPUT:	listHndl
	**	RESULT:	OSErr
	**
	**	Given a list record, this function places all of the zone names in
	**	the list.  Prior to calling this function, you need to create a
	**	list record that has zero rows and one column.  The zones are added
	**	to the list in the column in alphabetical order.  The function
	**	IUMagString is used to determine order. */

OSErr			HiliteUserZone(ListHandle listHndl);
	/*
	**	 Hilite current zone in zone list.
	**
	**	OUTPUT:	listHndl
	**	RESULT:	OSErr
	**	
	**	Given a list of zones (probably generated by DoBuildZoneList), this
	**	function selects the zone the user is currently in.  The current zone
	**	is selected and scrolled into view. */

OSErr			AddPPCNBPAlias(NamesTableEntry *theNTE, Str32 newNBPType, EntityName *newEntity);
	/*
	**	 Register network PPC alias.
	**
	**	INPUT:	newNBPType	The alias to register for this process
	**	OUTPUT:	theNTE		Receives the resultant NamesTableEntry
	**			newEntity	Receives the resultalt EntityName for use by RemoveNBPAlias
	**	RESULT:	OSErr		If an error is returned, RemoveNBPAlias should not be called.
	**						Usually, calls to AddPPCNBPAlias and Remove NBPAlias are
	**						balanced, but this depends on success by ADDPPCNBPAlias.
	**
	**	This function registers an alias for the application.  This allows PPCBrowser
	**	to be used to display only machines registered with that alias, instead of
	**	all machines that are PPCToolbox capable.  This function is called by 
	**	the application at startup time, as the alias only needs to be registered once.
	**	If there is more than one version of the application running on a single
	**	machine, AddPPCNBPAlias will append a number to the application name.
	**	This allows each copy of the application to be displayed by PPCBrowser.
	**
	**	A sample usage of this function can be found in DTS.StyleChat or Kibitz, in the
	**	file Start.c. */

OSErr			RemoveNBPAlias(EntityPtr theEntity);
	/*
	**	 Remove previously registered PPC alias.
	**
	**	INPUT:	theEntity	Pass in the output from AddPPCNBPAlias
	**	RESULT:	OSErr
	**
	**	This function removes the designated alias.  This is executed by the application
	**	at application quit time.  Note that it is very important to remove the NBPAlias,
	**	as it will remain in the queue even after the application has quit.  So don't just
	**	call ExitToShell to exit your application if you are using an NBPAlias.  Remove it
	**	first.  (This presents a problem during development.  If your application crashes,
	**	the application will have quit qithout removing the NBPAlias.  This will cause your
	**	machine to crash shortly.)
	**
	**	A sample usage of this function can be found in DTS.StyleChat or Kibitz, in the
	**	file Start.c. */


#endif
